
/*
   LibMatrixEx1.mq4
   Copyright  2009 Logunov Eugene (lea)
   rootrat@mail.ru
*/

#property copyright "Copyright  2009 Logunov Eugene (lea)"
#property link "rootrat@mail.ru"

//      
#property show_inputs

//  
#include <LibMatrix.mqh>

//   
extern int delay = 100;
//  
extern int degree = 5;
//     
extern int linesMargin = 15;
//   
extern int linesWidth = 3;
//     
extern color colVLineInt = Red;
//      
extern color colVLineExtr = Green;
//      
extern color colInt = DimGray;
//      
extern color colExt = DarkGray;

//   (  X  Y)
double SXY(double& x[], double& y[], int numPoints, int dx, int dy) {
   double s = 0;
   for (int i = 0; i < numPoints; i++) {
      s += MathPow(x[i], dx) * MathPow(y[i], dy);
   }
   return (s / numPoints);
}

//   
bool Regression(double& x[], double& y[], int numPoints, int polyDegree, double& poly[]) {
   //   
   double A[];
   int numRowsA = polyDegree + 1;
   int numColsA = polyDegree + 1;
   MatrSetSize(A, numRowsA, numColsA);
   //  
   for (int row = 0; row < numRowsA; row++) {
      for (int col = 0; col < numColsA; col++) {
         int offset = MatrIndiciesToOffset(row, col, numRowsA, numColsA);
         A[offset] = SXY(x, y, numPoints, row + col, 0);
      }
   }
   //    
   double B[];
   int numRowsB = polyDegree + 1;
   int numColsB = 1;
   MatrSetSize(B, numRowsB, numColsB);
   //    
   for (row = 0; row < numRowsB; row++) {
      offset = MatrIndiciesToOffset(row, 0, numRowsB, numColsB);
      B[offset] = SXY(x, y, numPoints, row, 1);
   }
   //  
   int numRowsX, numColsX;
   bool status =
      MatrGJBatchSolve(
         A, numRowsA, numColsA,
         B, numRowsB, numColsB,
         poly, numRowsX, numColsX,
         DEFAULT_TOLERANCE
      );
   if (!status) {
      Print("    ");
   }
   return (status);
}

//     
double EvalPoly(double x,double& poly[],int degree) {
   double value = poly[degree];
   for (int i=degree - 1; i >= 0; i--) {
      value = (value * x) + poly[i];
   }
   return (value);
}

//    
bool initOk = true;
//    
int numLines = 0;

//  
int init() {
   //          
   if (!ObjectCreate("RegTime_End", OBJ_VLINE, 0, WindowTimeOnDropped(), 0)) {
      Print("     RegTime_End");
      initOk = false;
      return (0);
   }
   if (!ObjectCreate("RegTime_Start", OBJ_VLINE, 0, WindowTimeOnDropped() - linesMargin * (Period() * 60), 0)) {
      ObjectDelete("RegTime_End");
      Print("     RegTime_Start");
      initOk = false;
      return (0);
   }
   if (!ObjectCreate("RegTime_ExtEnd", OBJ_VLINE, 0, WindowTimeOnDropped() + linesMargin * (Period() * 60), 0)) {
      ObjectDelete("RegTime_End");
      ObjectDelete("RegTime_Start");
      Print("     RegTime_ExtEnd");
      initOk = false;
      return (0);
   }
   //    
   ObjectSet("RegTime_End", OBJPROP_COLOR, colVLineInt);
   ObjectSet("RegTime_Start", OBJPROP_COLOR, colVLineInt);
   ObjectSet("RegTime_ExtEnd", OBJPROP_COLOR, colVLineExtr);
   //  
   WindowRedraw();
   return (0);
}

//  
int deinit() {
   //     - 
   if (!initOk) {
      return (0);
   }
   //    
   for (int i = 0; i < numLines; i++) {
      ObjectDelete("RegPlot_" + i);
   }
   //   
   ObjectDelete("RegTime_End");
   ObjectDelete("RegTime_Start");
   ObjectDelete("RegTime_ExtEnd");
   //  
   WindowRedraw();
   return (0);
}

//    ()
int start() {
   //     - 
   if (!initOk) {
      return (0);
   }
   //    
   datetime timeEndPrev = -1;
   datetime timeStartPrev = -1;
   datetime timeExtEndPrev = -1;
   while (!IsStopped()) {
      //       
      Sleep(delay);
      //       (  - )
      if (ObjectFind("RegTime_End") == -1) {
         Print("   RegTime_End");
         break;
      }
      if (ObjectFind("RegTime_Start") == -1) {
         Print("   RegTime_Start");
         break;
      }
      if (ObjectFind("RegTime_ExtEnd") == -1) {
         Print("   RegTime_ExtEnd");
         break;
      }
      //    
      datetime timeEnd = ObjectGet("RegTime_End", OBJPROP_TIME1);
      datetime timeStart = ObjectGet("RegTime_Start", OBJPROP_TIME1);
      datetime timeExtEnd = ObjectGet("RegTime_ExtEnd", OBJPROP_TIME1);
      //    
      if ((timeEnd == timeEndPrev) && (timeStart == timeStartPrev) && (timeExtEnd == timeExtEndPrev)) {
         continue;
      }
      timeEndPrev = timeEnd;
      timeStartPrev = timeStart;
      timeExtEndPrev = timeExtEnd;
      //   
      for (int i = 0; i < numLines; i++) {
         ObjectDelete("RegPlot_" + i);
      }
      numLines = 0;
      //   
      int shiftEnd = iBarShift(Symbol(), Period(), timeEnd);
      int shiftStart = iBarShift(Symbol(), Period(), timeStart);
      int shiftExtEnd = iBarShift(Symbol(), Period(), timeExtEnd);
      //       
      int numBarsInt = shiftStart - shiftEnd + 1;
      int numBarsExt = shiftEnd - shiftExtEnd + 1;
      //  
      double x[];
      double y[];
      ArrayResize(x, numBarsInt);
      ArrayResize(y, numBarsInt);
      for (i = 0; i < numBarsInt; i++) {
         x[i] = i;
         y[i] = Close[shiftEnd + i];
      }
      //  
      double poly[];
      if (Regression(x, y, numBarsInt, degree, poly)) {
         //     
         for (i = 0; i < numBarsInt - 1; i++) {
            ObjectCreate(
               "RegPlot_" + numLines,
               OBJ_TREND,
               0,
               iTime(Symbol(), Period(), shiftEnd + i),
               EvalPoly(i, poly, degree),
               iTime(Symbol(), Period(), shiftEnd + i + 1),
               EvalPoly(i + 1, poly, degree)
               );
            ObjectSet(
               "RegPlot_" + numLines,
               OBJPROP_RAY,
               false
               );
            ObjectSet(
               "RegPlot_" + numLines,
               OBJPROP_COLOR,
               colInt
               );
            ObjectSet(
               "RegPlot_" + numLines,
               OBJPROP_WIDTH,
               linesWidth
               );
            numLines++;
         }
         //     
         for (i = -1; i > -numBarsExt; i--) {
            ObjectCreate(
               "RegPlot_" + numLines,
               OBJ_TREND,
               0,
               iTime(Symbol(), Period(), shiftEnd + i),
               EvalPoly(i, poly, degree),
               iTime(Symbol(), Period(), shiftEnd + i + 1),
               EvalPoly(i + 1, poly, degree)
               );
            ObjectSet(
               "RegPlot_" + numLines,
               OBJPROP_RAY,
               false
               );
            ObjectSet(
               "RegPlot_" + numLines,
               OBJPROP_COLOR,
               colExt
               );
            ObjectSet(
               "RegPlot_" + numLines,
               OBJPROP_WIDTH,
               linesWidth
               );
            numLines++;
         }
      }
      //  
      WindowRedraw();
   }
   return (0);
}

